import { defineStore } from "pinia";
import type {
  EquipmentTypeViewModel,
  CreateEquipmentTypeViewModel,
  UpdateEquipmentTypeViewModel,
} from "@/viewmodels/admin/unit/equipmentType.models";
import { http } from "@/serverCom";
import type { AxiosResponse } from "axios";

export const useEquipmentTypeStore = defineStore("equipmentType", {
  state: () => {
    return {
      equipmentTypes: [] as Array<EquipmentTypeViewModel & { tab_pos: number }>,
      totalCount: 0 as number,
      loading: "loading" as "loading" | "fetched" | "failed",
    };
  },
  actions: {
    fetchEquipmentTypes(offset = 0, count = 25, search = "", clear = false) {
      if (clear) this.equipmentTypes = [];
      this.loading = "loading";
      http
        .get(`/admin/equipmentType?offset=${offset}&count=${count}${search != "" ? "&search=" + search : ""}`)
        .then((result) => {
          this.totalCount = result.data.total;
          result.data.equipments
            .filter((elem: EquipmentTypeViewModel) => this.equipmentTypes.findIndex((m) => m.id == elem.id) == -1)
            .map((elem: EquipmentTypeViewModel, index: number): EquipmentTypeViewModel & { tab_pos: number } => {
              return {
                ...elem,
                tab_pos: index + offset,
              };
            })
            .forEach((elem: EquipmentTypeViewModel & { tab_pos: number }) => {
              this.equipmentTypes.push(elem);
            });
          this.loading = "fetched";
        })
        .catch((err) => {
          this.loading = "failed";
        });
    },
    async getAllEquipmentTypes(): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/equipment?noLimit=true`).then((res) => {
        return { ...res, data: res.data.equipments };
      });
    },
    async getEquipmentTypesByIds(ids: Array<string>): Promise<AxiosResponse<any, any>> {
      return await http
        .post(`/admin/equipment/ids`, {
          ids,
        })
        .then((res) => {
          return { ...res, data: res.data.equipments };
        });
    },
    async searchEquipmentTypes(search: string): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/equipment?search=${search}&noLimit=true`).then((res) => {
        return { ...res, data: res.data.equipments };
      });
    },
    fetchEquipmentTypeById(id: string) {
      return http.get(`/admin/equipment/${id}`);
    },
    fetchEquipmentTypeStatisticsById(id: string) {
      return http.get(`/admin/equipment/${id}/statistics`);
    },
    async createEquipmentType(equipment: CreateEquipmentTypeViewModel): Promise<AxiosResponse<any, any>> {
      const result = await http.post(`/admin/equipment`, {
        salutationId: equipment.salutationId,
        firstname: equipment.firstname,
        lastname: equipment.lastname,
        nameaffix: equipment.nameaffix,
        birthdate: equipment.birthdate,
        internalId: equipment.internalId,
      });
      this.fetchEquipmentTypes();
      return result;
    },
    async updateEquipmentType(equipment: UpdateEquipmentTypeViewModel): Promise<AxiosResponse<any, any>> {
      const result = await http.patch(`/admin/equipment/${equipment.id}`, {
        salutationId: equipment.salutationId,
        firstname: equipment.firstname,
        lastname: equipment.lastname,
        nameaffix: equipment.nameaffix,
        birthdate: equipment.birthdate,
        internalId: equipment.internalId,
      });
      this.fetchEquipmentTypes();
      return result;
    },
    async deleteEquipmentType(equipment: number): Promise<AxiosResponse<any, any>> {
      const result = await http.delete(`/admin/equipment/${equipment}`);
      this.fetchEquipmentTypes();
      return result;
    },
  },
});
